@@ -1,4 +1,4 @@
-# contextdiff.__main__
+# net_contextdiff.__main__
 
 
 # Copyright (C) Robert Franklin <rcf34@cam.ac.uk>
@@ -8,16 +8,16 @@
 
 
 
-import net_contextdiff
+from net_contextdiff import __version__
 
 import argparse
 import re
 import sys
 
+from deepops import deepmerge
 import yaml
 
-from net_contextdiff.iosparser import CiscoIOSConfig
-from net_contextdiff.iosdiff import CiscoIOSDiffConfig
+from .ios import CiscoIOSConfig, CiscoIOSDiffConfig
 
 
 
@@ -39,6 +39,107 @@ PLATFORMS = {
 
 
 
+# --- functions ---
+
+
+
+def diffconfig(diff, devicename, from_filename, to_filename, output_filename,
+               no_output, dump_rules, debug_parser, debug_convert):
+
+    """This function compares the 'from' and 'to' configurations for
+    the specified device and writes a difference configuration file
+    (one that transforms the configuration of a running device from the
+    'from' to the 'to' state).
+
+    The filenames for the configuration files, as well as the output
+    are taken from the arguments parsed above and stored in global
+    variables used directly by this function.
+
+    The function returns True iff the parsing and comparison succeeded.
+    Most problems result in the function aborting with an exception,
+    but some minor warnings may be ignored and the program continue
+    with devices.
+
+    Keyword arguments:
+
+    diff -- the DiffConfig object to do the convert the configuration
+
+    devicename -- the name of the device being converted, or None if
+    it's not known
+
+    from_filename -- filename of the source configuration
+
+    to_filename -- filename of the target configuration
+
+    output_filename -- name of the file to write the conversion
+    configuration to or None for standard output
+
+    no_output -- a boolean set to True iff no output file is to be
+    written
+
+    dump_rules, debug_parser, debug_convert -- the equivalent
+    variables to command line arguments
+    """
+
+
+    # reset the rules list for this conversion - this is the same
+    # list each time, except the devicename changes and may cause
+    # some of the rules to change
+
+    diff.init_rules_active()
+    diff.add_rules_active(rule_specs, devicename)
+
+    if dump_rules >= 2:
+        num = 0
+        for spec, tree in diff.get_rules():
+            print(">> rule num: %d spec: %s" % (num, spec))
+            print(yaml.dump(tree, default_flow_style=False)
+                        if tree else "<none>\n", sep="\n")
+
+            num += 1
+
+
+    # read in the 'from' and 'to' configurations
+
+    from_cfg = config_parser_class(filename=from_filename, debug=debug_parser)
+
+    to_cfg = (config_parser_class(filename=to_filename, debug=debug_parser)
+                  if to_filename else None)
+
+
+    # find the differences
+
+    diffs, diffs_tree = diff.convert(from_cfg, to_cfg)
+
+
+    # if output is suppressed, just stop with success here
+
+    if no_output:
+        return diffs_tree
+
+
+    # write the differences to a file or standard output
+
+    if output_filename:
+        if debug_convert:
+            print("debug: writing to output file:", output_filename)
+
+        with open(output_filename, "w") as output_file:
+            if diffs:
+                print(diffs, file=output_file)
+
+    else:
+        if debug_convert:
+            print("debug: writing to standard output")
+
+        if diffs:
+            print(diffs)
+
+
+    return diffs_tree
+
+
+
 # --- command line arguments ---
 
 
@@ -53,20 +154,7 @@ parser = argparse.ArgumentParser(
 
     # we want the epilog help output to be printed as it and not reformatted or
     # line wrapped
-    formatter_class=argparse.RawDescriptionHelpFormatter,
-
-    epilog="""\
-Exclude items are added using the -x and -y options: they consist of paths
-through the configuration structure with levels separated by colons.  For
-example, excluding "interface:Vlan100" will avoid changes to the VLAN 100
-interface.
-
-The -e option is useful to add comments to conversions to indicate what path
-matched a particular conversion, so excludes can be constructed.
-
-Note that exclude items are case-sensitive and must be matched exactly.
-""")
-
+    formatter_class=argparse.RawDescriptionHelpFormatter)
 
 parser.add_argument(
     "-r", "--rules-tree-filename",
@@ -104,13 +192,30 @@ parser.add_argument(
          "exclude and conversion process")
 
 parser.add_argument(
-    "-R", "--debug-rules",
+    "-R", "--dump-rules",
     action="count",
     default=0,
     help="dump the tree of items to be included and excluded from the "
          "comparisons, based on the rules (multiple levels increase "
          "verbosity up to a maximum of 2)")
 
+parser.add_argument(
+    "-C", "--dump-config",
+    action="store_true",
+    help="dump the configuration dictionary after parsing")
+
+parser.add_argument(
+    "-D", "--dump-diff",
+    action="store_true",
+    help="dump the difference (removes and updates) between the from "
+         "and to configurations")
+
+parser.add_argument(
+    "-T", "--dump-diff-tree",
+    action="store_true",
+    help="dump the differences across all compared files as a YAML "
+         "tree - this is useful for using in an exclude file")
+
 parser.add_argument(
     "-P", "--debug-parser",
     action="count",
@@ -118,17 +223,6 @@ parser.add_argument(
     help="enable debugging of contextual parsing (multiple levels "
          "increase verbosity up to a maxmimum of 2)")
 
-parser.add_argument(
-    "-C", "--debug-config",
-    action="store_true",
-    help="dump the configuration dictionary after parsing")
-
-parser.add_argument(
-    "-D", "--debug-diff",
-    action="store_true",
-    help="dump the difference (removes and updates) between the from "
-         "and to configurations")
-
 parser.add_argument(
     "-V", "--debug-convert",
     action="count",
@@ -173,9 +267,9 @@ parser.add_argument(
          "configuration for")
 
 parser.add_argument(
-    "--version",
+    "-v", "--version",
     action="version",
-    version=("%(prog)s " + net_contextdiff.__version__))
+    version=("%(prog)s " + __version__))
 
 
 # parse the supplied command line against these options, storing the results
@@ -189,10 +283,11 @@ rule_specs = args.rule_spec
 explain = args.explain
 quiet = args.quiet
 no_output = args.no_output
-debug_rules = args.debug_rules
+dump_rules = args.dump_rules
+dump_config = args.dump_config
+dump_diff = args.dump_diff
+dump_diff_tree = args.dump_diff_tree
 debug_parser = args.debug_parser
-debug_config = args.debug_config
-debug_diff = args.debug_diff
 debug_convert = args.debug_convert
 from_filename = args.from_filename
 to_filename = args.to_filename
@@ -207,7 +302,7 @@ devicenames = args.devicenames
 if not rule_specs:
     rule_specs = [
         "!rules:exclude:ALL",
-        "'!rules:exclude:%",
+        "!rules:exclude:%",
         "rules:include:ALL",
         "rules:include:%",
     ]
@@ -216,25 +311,30 @@ if not rule_specs:
 # check a couple of nonsensical configurations aren't being use related
 # to multiple devices
 
-if len(devicenames) == 0:
+if not len(devicenames):
     if from_filename.find("%") != -1:
-        print("warning: no device names specified, so operating on a single "
-              "file, yet 'from' filename has '%' character - no substitution "
-              "will be performed",
+        print("error: no device names specified, so operating on a single "
+              "file, yet 'from' filename has '%' character",
               file=sys.stderr)
 
+        exit(1)
+
+
     if to_filename and (to_filename.find("%") != -1):
-        print("warning: no device names specified, so operating on a single "
-              "file, yet 'to' filename has '%' character - no substitution "
-              "will be performed",
+        print("error: no device names specified, so operating on a single "
+              "file, yet 'to' filename has '%' character",
               file=sys.stderr)
 
+        exit(1)
+
+
     if output_filename and (output_filename.find("%") != -1):
-        print("warning: no device names specified, so operating on a single "
-              "file, yet 'output' filename has '%' character - no "
-              "substitution will be performed",
+        print("error: no device names specified, so operating on a single "
+              "file, yet 'output' filename has '%' character",
               file=sys.stderr)
 
+        exit(1)
+
 
 elif len(devicenames) > 1:
     if from_filename.find("%") == -1:
@@ -268,7 +368,7 @@ elif len(devicenames) > 1:
 
 # create the diff object once at the start
 
-diff = config_diff_class(explain, debug_config, debug_diff, debug_convert)
+diff = config_diff_class(explain, dump_config, dump_diff, debug_convert)
 
 
 # if a rules tree file was specified, read that in
@@ -280,7 +380,7 @@ if rules_tree_filename:
 # dump the rule tree, if debugging enabled for it (this will include any
 # rules included by default, in the class)
 
-if debug_rules >= 1:
+if dump_rules >= 1:
     print(">> rule tree:")
     print(yaml.dump(diff.get_rules_tree(), default_flow_style=False))
     print(">> rule specs:")
@@ -295,109 +395,49 @@ if debug_rules >= 1:
 
 
 
-def diffconfig(from_filename, to_filename, output_filename=None,
-               no_output=False):
-
-    """This function compares the 'from' and 'to' configurations for
-    the specified device and writes a difference configuration file
-    (one that transforms the configuration of a running device from the
-    'from' to the 'to' state).
-
-    The filenames for the configuration files, as well as the output
-    are taken from the arguments parsed above and stored in global
-    variables used directly by this function.
-
-    The function returns True iff the parsing and comparison succeeded.
-    Most problems result in the function aborting with an exception,
-    but some minor warnings may be ignored and the program continue
-    with devices.
-    """
-
-    # this function makes use of global variables defined outside it so
-    # must appear here in the code
-
-
-    # read in the 'from' and 'to' configurations
-
-    from_cfg = config_parser_class(from_filename, debug_parser)
-
-    to_cfg = (config_parser_class(to_filename, debug_parser)
-                  if to_filename else None)
-
-
-    # find the differences
-
-    diffs = diff.convert(from_cfg, to_cfg)
-
-
-    # if output is suppressed, just stop with success here
-
-    if no_output:
-        return True
-
-
-    # write the differences to a file or standard output
-
-    if output_filename:
-        if debug_convert:
-            print("debug: writing to output file:", output_filename)
-
-        with open(output_filename, "w") as output_file:
-            if diffs:
-                print(diffs, file=output_file)
-
-    else:
-        if debug_convert:
-            print("debug: writing to standard output")
-
-        if diffs:
-            print(diffs)
-
-
-    return True
-
-
-
-# this flag will change to False if any configuration fails to generate and
-# is used to affect the return code from the script
-
-complete_success = True
-
+# if we have a list of device names (even just one), we iterate through
+# them, else we just process the one file we've been given
 
 if devicenames:
+    # initialise the tree of aggregated differences across al the
+    # compared configuration files
+
+    all_diffs_tree = {}
+
+
     for devicename in devicenames:
         if not quiet:
             print(devicename)
 
 
-        # reset the rules list for this conversion - this is the same
-        # list each time, except the devicename changes and may cause
-        # some of the rules to change
-
-        diff.init_rules_active()
-        diff.add_rules_active(rule_specs, devicename)
-
-        if debug_rules >= 2:
-            num = 0
-            for spec, tree in diff.get_rules():
-                print(">> rule num: %d spec: %s" % (num, spec))
-                print(yaml.dump(tree, default_flow_style=False)
-                          if tree else "<none>\n", sep="\n")
-
-                num += 1
-
-
         # do the actual conversion
 
-        complete_success = diffconfig(
-                               from_filename.replace("%", devicename),
-                               to_filename.replace("%", devicename),
-                               output_filename.replace("%", devicename))
+        diffs_tree = diffconfig(
+                      diff, devicename,
+                      from_filename.replace("%", devicename),
+                      to_filename.replace("%", devicename),
+                      output_filename.replace("%", devicename),
+                      no_output, dump_rules, debug_parser,
+                      debug_convert)
+
+
+        # add the differences in this conversion to the aggregate tree
+
+        deepmerge(all_diffs_tree, diffs_tree)
 
 
 else:
-    complete_success = diffconfig(
-        from_filename, to_filename, output_filename, no_output)
+    all_diffs_tree = diffconfig(
+        diff, None, from_filename, to_filename, output_filename, no_output,
+        dump_rules, debug_parser, debug_convert)
 
 
-exit(0 if complete_success else 1)
+# if the option to enable dumping of the differences tree is enabled, do
+# that
+
+if dump_diff_tree:
+    print("=> differences tree:")
+    print(yaml.dump(all_diffs_tree, default_flow_style=False))
+
+
+exit(0)
