@@ -86,7 +86,7 @@ class DiffConvert:
 
     cmd -- the path for this command in the configuration, following
     the context; this part of the dictionary will be passed in the
-    'old/new' and 'rem/upddiff' parameters to the action methods; this
+    'old/new' and 'rem/upd' parameters to the action methods; this
     variable defaults to None and must be defined by a child class
 
     ext -- an extension to path which must be matched, but this part is
@@ -134,9 +134,10 @@ class DiffConvert:
             i for i, v in enumerate(self._path_full) if v is None }
 
 
-    def _pathmatches(self, d, path):
-        """This function is used to recursively step along the
-        paths.
+    def _path_matches(self, d, path):
+        """This method is used to recursively step along the
+        paths.  It is initially called by full_matches() from the top of
+        the path - see that for more information.
         """
 
         # if the path is finished, return a single result with an
@@ -170,7 +171,7 @@ class DiffConvert:
                 if path_tail:
                     # yes - recursively get the matches from under
                     # this key
-                    for matches in self._pathmatches(d[d_key], path_tail):
+                    for matches in self._path_matches(d[d_key], path_tail):
                         # add this match to the list of matches,
                         # prepending this key onto the start of the
                         # path
@@ -188,7 +189,7 @@ class DiffConvert:
         if path_head in d:
             return [ ([path_head] + matches)
                             for matches
-                            in self._pathmatches(d[path_head], path_tail) ]
+                            in self._path_matches(d[path_head], path_tail) ]
 
         # we have no matches, return the empty list
         return []
@@ -209,11 +210,11 @@ class DiffConvert:
         particular order.
         """
 
-        return self._pathmatches(d, self._path_full)
+        return self._path_matches(d, self._path_full)
 
 
     def context_removed(self, d, match):
-        """This method take a remove dictionary, as returned by
+        """This method takes a remove dictionary, as returned by
         deepdiff(), and a specific match into it (which must be from
         the same DiffConvert object as the method is called on) and
         returns True iff the context for this converter is in it
@@ -241,12 +242,51 @@ class DiffConvert:
             return False
 
 
-    def remove(self, old, rem, *args):
+    def remove(self, old, *args):
         """The remove() method is called when the specified path is in
         the remove differences (i.e. it's in the 'from' configuration
         but not in the 'to' configuration), unless the containing
         context is being removed in its entirety.
 
+        The default behaviour of this method is to call truncate() with
+        the old value for 'old' and 'remove' as, if there is no specific
+        behaviour for removing an entire object, we should remove the
+        individual elements.
+
+        Keyword arguments:
+
+        old -- the value of the dictionary item at the matching path in
+        the 'from' configuration dictionary (sometimes, the full details
+        of the old configuration may be required to remove it)
+
+        *args -- [only] the arguments in the path which were matched
+        with wildcards, supplied in the order they were given in the
+        context, cmd and ext (e.g. for a match of ["interface", None,
+        "standby", None], this will be second and fouth elements of
+        the path); if there are no wildcards, no extra arguments will
+        be supplied
+
+        The return value is the commands to insert into the
+        configuration to convert it.  This can either be a simple
+        string, in the case of only one line being required, or an
+        iterable containing one string per line.  If the return value is
+        None, it is an indication nothing needed to be done.  An empty
+        string or iterable indicates the update did something, which did
+        not change the configuration (this may have semantic
+        differences).
+        """
+
+        return self.truncate(old, old, *args)
+
+
+    def truncate(self, old, rem, *args):
+        """The truncate() method is identical to remove() except that it
+        is called when an item is partially removed (i.e. something
+        remains in the 'to' configuration - it is 'truncated').
+
+        It is useful when amending lists or sets and matching at the
+        containing object level.
+
         Keyword arguments:
 
         old -- the value of the dictionary item at the matching path in
@@ -277,11 +317,50 @@ class DiffConvert:
         pass
 
 
+    def add(self, new, *args):
+        """The add() method is called when the specified path is in the
+        update differences but did not exist in the 'from' configuration
+        (i.e. it's something new that is being added to the
+        configuration).
+
+        By default, this calls update() with the new configuration as
+        the updated and new configuration arguments (and 'args') as, in
+        many cases, the process for adding something is the same as
+        updating it (e.g. updating an interface description).  It can,
+        however, be overridden to do something different or, more
+        commonly, implement add() but not update() for a particular
+        change (as the updates will be picked up by more specific
+        paths in other objects).
+
+        Keyword arguments:
+
+        new -- the value of the dictionary item at the matching path in
+        the 'to' configuration dictionary
+
+        *args -- [only] the arguments in the path which were matched
+        with wildcards, supplied in the order they were given in the
+        context, cmd and ext (e.g. for a match of ["interface", None,
+        "standby", None], this will be second and fouth elements of
+        the path); if there are no wildcards, no extra arguments will
+        be supplied
+
+        The return value is the commands to insert into the
+        configuration to convert it.  This can either be a simple
+        string, in the case of only one line being required, or an
+        iterable containing one string per line.  If the return value is
+        None, it is an indication nothing needed to be done.  An empty
+        string or iterable indicates the update did something, which did
+        not change the configuration (this may have semantic
+        differences).
+        """
+
+        return self.update(None, new, new, *args)
+
+
     def update(self, old, upd, new, *args):
         """The update() method is called when the specified path is in
-        the update differences (i.e. it's in the 'to' configuration but
-        not in the 'from' configuration, or the value differs between
-        the two and needed to be changed).
+        the update differences and also in the 'from' configuration
+        (i.e. something is being updated in the configuration).
 
         Keyword arguments:
 
@@ -294,7 +373,7 @@ class DiffConvert:
         the update differences dictionary
 
         new -- the value of the dictionary item at the matching path in
-        the 'to' configuration dictionary (sometimes, teh full details
+        the 'to' configuration dictionary (sometimes, the full details
         of the new configuration may be required to update it)
 
         *args -- [only] the arguments in the path which were matched
@@ -807,15 +886,32 @@ class DiffConfig:
 
                     if cvt.context_removed(remove_cfg, match):
                         if self._debug_convert:
-                            print(
-                                "-> containing context being removed - skip")
+                            print("-> containing context being removed - skip")
 
                         continue
 
 
-                    # get elements in the path matching wildcards
+                    # check if anything remains at this level or below
+                    # in the 'to' configuration - if it does, we're
+                    # doing a partial removal
 
-                    args = cvt.get_wildcard_args(match)
+                    remove_is_trunc = False
+
+                    try:
+                        # we don't want the result here, just to find
+                        # out if the path exists (if not, KeyError will
+                        # be raised)
+                        deepget(to_cfg, *match, default_error=True)
+
+                    except KeyError:
+                        # nothing remains so we're doing a full remove
+                        pass
+
+                    else:
+                        # something remains so this is partial
+                        if self._debug_convert:
+                            print("-> subconfiguration not empty - truncate")
+                        remove_is_trunc = True
 
 
                     # get the 'from' and 'remove' parts of the
@@ -837,11 +933,19 @@ class DiffConfig:
                             sep="\n")
 
 
+                    # get elements in the path matching wildcards
+
+                    args = cvt.get_wildcard_args(match)
+
+
                     # call the remove converter action method using the
                     # corresponding entry from the 'from' configuration
 
                     try:
-                        diff = cvt.remove(cvt_old, cvt_rem, *args)
+                        if remove_is_trunc:
+                            diff = cvt.truncate(cvt_old, cvt_rem, *args)
+                        else:
+                            diff = cvt.remove(cvt_old, *args)
 
                     except:
                         print("with: type(cvt)=" + repr(type(cvt)),
@@ -902,6 +1006,22 @@ class DiffConfig:
                     args = cvt.get_wildcard_args(match)
 
 
+                    # check if there is anything for this level in the
+                    # 'from' configuration - if not, we're actually
+                    # adding this, rather than updating it, so record
+                    # that
+
+                    update_is_add = False
+
+                    try:
+                        deepget(from_cfg, *match, default_error=True)
+
+                    except KeyError:
+                        if self._debug_convert:
+                            print("-> no old configuration - add")
+                        update_is_add = True
+
+
                     # get the 'from' and 'update' parts of the
                     # configuration and update difference dictionaries,
                     # for the path specified in the converter (ignoring
@@ -928,7 +1048,12 @@ class DiffConfig:
                     # call the update converter action method
 
                     try:
-                        diff = cvt.update(cvt_old, cvt_upd, cvt_new, *args)
+                        # if we're adding this, call the add() method,
+                        # else update()
+                        if update_is_add:
+                            diff = cvt.add(cvt_new, *args)
+                        else:
+                            diff = cvt.update(cvt_old, cvt_upd, cvt_new, *args)
 
                     except:
                         print("with: type(cvt)=" + repr(type(cvt)),
@@ -937,6 +1062,8 @@ class DiffConfig:
                         raise
 
 
+                    # (same as in remove, above)
+
                     if diff is not None:
                         if isinstance(diff, str):
                             diff = [diff]
