@@ -126,10 +126,10 @@ cmds.append(_Cmd_Int_CDPEna)
 
 
 class _Cmd_Int_ChnGrp(_CmdContext_Int):
-    match = r"channel-group (?P<ch_grp>\d+)(?P<mode> .+)?"
+    match = r"channel-group (?P<id_>\d+)(?P<mode> .+)?"
 
-    def parse(self, cfg, ch_grp, mode):
-        cfg["channel-group"] = int(ch_grp), mode
+    def parse(self, cfg, id_, mode):
+        cfg["channel-group"] = int(id_), mode
 
 cmds.append(_Cmd_Int_ChnGrp)
 
@@ -334,7 +334,8 @@ class _Cmd_Int_StandbyIPSec(_CmdContext_Int):
 
     def parse(self, cfg, grp, addr):
         deepsetdefault(
-            cfg, "standby", "group", int(grp), "ip-sec", last=set()).add(addr)
+            cfg, "standby", "group", int(grp), "ip-secondary",
+            last=set()).add(addr)
 
 cmds.append(_Cmd_Int_StandbyIPSec)
 
@@ -398,11 +399,47 @@ class _Cmd_Int_StandbyVer(_CmdContext_Int):
 cmds.append(_Cmd_Int_StandbyVer)
 
 
+class _Cmd_Int_StormCtrl(_CmdContext_Int):
+    match = r"storm-control (?P<traffic>\S+) level (?P<level>[0-9.]+)"
+
+    def parse(self, cfg, traffic, level):
+        deepsetdefault(cfg, "storm-control")[traffic] = float(level)
+
+cmds.append(_Cmd_Int_StormCtrl)
+
+
+class _Cmd_Int_SwPort(_CmdContext_Int):
+    match = r"(?P<no>no )?switchport"
+
+    def parse(self, cfg, no):
+        cfg["switchport"] = not no
+
+cmds.append(_Cmd_Int_SwPort)
+
+
+class _Cmd_Int_SwPortMode(_CmdContext_Int):
+    match = r"switchport mode (?P<mode>\S+)"
+
+    def parse(self, cfg, mode):
+        cfg["switchport-mode"] = mode
+
+cmds.append(_Cmd_Int_SwPortMode)
+
+
+class _Cmd_Int_SwPortNoNeg(_CmdContext_Int):
+    match = r"switchport nonegotiate"
+
+    def parse(self, cfg):
+        cfg["switchport-nonegotiate"] = True
+
+cmds.append(_Cmd_Int_SwPortNoNeg)
+
+
 class _Cmd_Int_SwPortTrkNtv(_CmdContext_Int):
     match = r"switchport trunk native vlan (?P<vlan>\d+)"
 
     def parse(self, cfg, vlan):
-        cfg["swport-trunk-native"] = int(vlan)
+        cfg["switchport-trunk-native"] = int(vlan)
 
 cmds.append(_Cmd_Int_SwPortTrkNtv)
 
