@@ -47,7 +47,7 @@ _cvts = []
 class _Cvt_Hostname(DiffConvert):
     cmd = "hostname",
 
-    def remove(self, old, rem):
+    def remove(self, old):
         return "no hostname"
 
     def update(self, old, upd, new):
@@ -64,18 +64,15 @@ _cvts.append(_Cvt_Hostname)
 class _Cvt_Int(DiffConvert):
     cmd = "interface", None
 
-    def remove(self, old, rem, int_name):
+    def remove(self, old, int_name):
         # only remove the interface if it's not physical
         if is_int_physical(int_name):
             return
 
-        # remove if interface no longer exists, or exists flag has gone
-        if (not rem) or ("exists" in rem):
-            return "no interface " + int_name
+        return "no interface " + int_name
 
-    def update(self, old, upd, new, int_name):
-        if "exists" in upd:
-            return "interface " + int_name
+    def add(self, new, int_name):
+        return "interface " + int_name
 
 _cvts.append(_Cvt_Int)
 
@@ -105,7 +102,7 @@ _cvts.append(_Cvt_Int_Shutdown)
 class _Cvt_Int_VRFFwd(_CvtContext_Int):
     cmd = "vrf-forwarding",
 
-    def remove(self, old, rem, int_name):
+    def remove(self, old, int_name):
         return "interface " + int_name, " no vrf forwarding"
 
     def update(self, old, upd, new, int_name):
@@ -119,7 +116,7 @@ _cvts.append(_Cvt_Int_VRFFwd)
 class _Cvt_Int_ARPTime(_CvtContext_Int):
     cmd = "arp-timeout",
 
-    def remove(self, old, rem, int_name):
+    def remove(self, old, int_name):
         return "interface " + int_name, " no arp timeout"
 
     def update(self, old, upd, new, int_name):
@@ -131,7 +128,7 @@ _cvts.append(_Cvt_Int_ARPTime)
 class _Cvt_Int_CDPEna(_CvtContext_Int):
     cmd = "cdp-enable",
 
-    def remove(self, old, rem, int_name):
+    def remove(self, old, int_name):
         # if the 'cdp enable' option is not present, that doesn't mean
         # it's disabled but just that it's not specified, so we assume
         # the default is for it to be enabled
@@ -147,7 +144,7 @@ _cvts.append(_Cvt_Int_CDPEna)
 class _Cvt_Int_ChnGrp(_CvtContext_Int):
     cmd = "channel-group",
 
-    def remove(self, old, rem, int_name):
+    def remove(self, old, int_name):
         return "interface " + int_name, " no channel-group"
 
     def update(self, old, upd, new, int_name):
@@ -161,7 +158,7 @@ _cvts.append(_Cvt_Int_ChnGrp)
 class _Cvt_Int_Desc(_CvtContext_Int):
     cmd = "description",
 
-    def remove(self, old, rem, int_name):
+    def remove(self, old, int_name):
         return "interface " + int_name, " no description"
 
     def update(self, old, upd, new, int_name):
@@ -173,7 +170,7 @@ _cvts.append(_Cvt_Int_Desc)
 class _Cvt_Int_Encap(_CvtContext_Int):
     cmd = "encapsulation",
 
-    def remove(self, old, rem, int_name):
+    def remove(self, old, int_name):
         return "interface " + int_name, " no encapsulation " + rem
 
     def update(self, old, upd, new, int_name):
@@ -183,19 +180,13 @@ _cvts.append(_Cvt_Int_Encap)
 
 
 class _Cvt_Int_IPAccGrp(_CvtContext_Int):
-    cmd = "ip-access-group",
+    cmd = "ip-access-group", None
 
-    def remove(self, old, rem, int_name):
-        l = ["interface " + int_name]
-        for dir_ in sorted(rem):
-            l.append(" no ip access-group " + dir_)
-        return l
+    def remove(self, old, int_name, dir_):
+        return "interface " + int_name, " no ip access-group " + dir_
 
-    def update(self, old, upd, new, int_name):
-        l = ["interface " + int_name]
-        for dir_ in sorted(upd):
-            l.append(" ip access-group %s %s" % (upd[dir_], dir_))
-        return l
+    def update(self, old, upd, new, int_name, dir_):
+        return "interface " + int_name, " ip access-group %s %s" % (upd, dir_)
 
 _cvts.append(_Cvt_Int_IPAccGrp)
 
@@ -203,7 +194,7 @@ _cvts.append(_Cvt_Int_IPAccGrp)
 class _Cvt_Int_IPAddr(_CvtContext_Int):
     cmd = "ip-address",
 
-    def remove(self, old, rem, int_name):
+    def remove(self, old, int_name):
         return "interface " + int_name, " no ip address"
 
     def update(self, old, upd, new, int_name):
@@ -213,19 +204,13 @@ _cvts.append(_Cvt_Int_IPAddr)
 
 
 class _Cvt_Int_IPAddrSec(_CvtContext_Int):
-    cmd = "ip-address-secondary",
+    cmd = "ip-address-secondary", None
 
-    def remove(self, old, rem, int_name):
-        l = ["interface " + int_name]
-        for addr in sorted(rem):
-            l.append(" no ip address %s secondary" % addr)
-        return l
+    def remove(self, old, int_name, addr):
+        return "interface " + int_name, " no ip address %s secondary" % addr
 
-    def update(self, old, upd, new, int_name):
-        l = ["interface " + int_name]
-        for addr in sorted(upd):
-            l.append(" ip address %s secondary" % addr)
-        return l
+    def update(self, old, upd, new, int_name, addr):
+        return "interface " + int_name, " ip address %s secondary" % addr
 
 _cvts.append(_Cvt_Int_IPAddrSec)
 
@@ -233,9 +218,9 @@ _cvts.append(_Cvt_Int_IPAddrSec)
 class _Cvt_Int_IPFlowMon(_CvtContext_Int):
     cmd = "ip-flow-monitor", None
 
-    def remove(self, old, rem, int_name, dir_):
+    def remove(self, old, int_name, dir_):
         return ("interface " + int_name,
-                " no ip flow monitor %s %s" % (rem, dir_))
+                " no ip flow monitor %s %s" % (old, dir_))
 
     def update(self, old, upd, new, int_name, dir_):
         l = ["interface " + int_name]
@@ -251,19 +236,13 @@ _cvts.append(_Cvt_Int_IPFlowMon)
 
 
 class _Cvt_Int_IPHlprAddr(_CvtContext_Int):
-    cmd = "ip-helper-address",
+    cmd = "ip-helper-address", None
 
-    def remove(self, old, rem, int_name):
-        l = ["interface " + int_name]
-        for a in sorted(rem):
-            l.append(" no ip helper-address " + a)
-        return l
+    def remove(self, old, int_name, addr):
+        return "interface " + int_name, " no ip helper-address " + addr
 
-    def update(self, old, upd, new, int_name):
-        l = ["interface " + int_name]
-        for a in sorted(upd):
-            l.append(" ip helper-address " + a)
-        return l
+    def update(self, old, upd, new, int_name, addr):
+        return "interface " + int_name, " ip helper-address " + addr
 
 _cvts.append(_Cvt_Int_IPHlprAddr)
 
@@ -271,7 +250,7 @@ _cvts.append(_Cvt_Int_IPHlprAddr)
 class _Cvt_Int_IPIGMPVer(_CvtContext_Int):
     cmd = "ip-igmp-version",
 
-    def remove(self, old, rem, int_name):
+    def remove(self, old, int_name):
         return "interface " + int_name, " no ip igmp version"
 
     def update(self, old, upd, new, int_name):
@@ -283,7 +262,7 @@ _cvts.append(_Cvt_Int_IPIGMPVer)
 class _Cvt_Int_IPMcastBdry(_CvtContext_Int):
     cmd = "ip-multicast-boundary",
 
-    def remove(self, old, rem, int_name):
+    def remove(self, old, int_name):
         return "interface " + int_name, " no ip multicast boundary"
 
     def update(self, old, upd, new, int_name):
@@ -295,7 +274,7 @@ _cvts.append(_Cvt_Int_IPMcastBdry)
 class _Cvt_Int_IPPIMMode(_CvtContext_Int):
     cmd = "ip-pim", "mode"
 
-    def remove(self, old, rem, int_name):
+    def remove(self, old, int_name):
         return "interface " + int_name, " no ip pim " + old
 
     def update(self, old, upd, new, int_name):
@@ -307,7 +286,7 @@ _cvts.append(_Cvt_Int_IPPIMMode)
 class _Cvt_Int_IPPIMBSRBdr(_CvtContext_Int):
     cmd = "ip-pim", "bsr-border"
 
-    def remove(self, old, rem, int_name):
+    def remove(self, old, int_name):
         return "interface " + int_name, " no ip pim bsr-border"
 
     def update(self, old, upd, new, int_name):
@@ -329,7 +308,7 @@ _cvts.append(_Cvt_Int_IPProxyARP)
 class _Cvt_Int_IPVerifyUni(_CvtContext_Int):
     cmd = "ip-verify-unicast",
 
-    def remove(self, old, rem, int_name):
+    def remove(self, old, int_name):
         return "interface " + int_name, " no ip verify unicast"
 
     def update(self, old, upd, new, int_name):
@@ -339,19 +318,13 @@ _cvts.append(_Cvt_Int_IPVerifyUni)
 
 
 class _Cvt_Int_IPv6Addr(_CvtContext_Int):
-    cmd = "ipv6-address",
+    cmd = "ipv6-address", None
 
-    def remove(self, old, rem, int_name):
-        l = ["interface " + int_name]
-        for a in sorted(rem):
-            l.append(" no ipv6 address " + a)
-        return l
+    def remove(self, old, int_name, addr):
+        return "interface " + int_name, " no ipv6 address " + addr
 
-    def update(self, old, upd, new, int_name):
-        l = ["interface " + int_name]
-        for a in sorted(upd):
-            l.append(" ipv6 address " + a)
-        return l
+    def update(self, old, upd, new, int_name, addr):
+        return "interface " + int_name, " ipv6 address " + addr
 
 _cvts.append(_Cvt_Int_IPv6Addr)
 
@@ -359,7 +332,7 @@ _cvts.append(_Cvt_Int_IPv6Addr)
 class _Cvt_Int_IPv6MultBdry(_CvtContext_Int):
     cmd = "ipv6-multicast-boundary-scope",
 
-    def remove(self, old, rem, int_name):
+    def remove(self, old, int_name):
         return "interface " + int_name, " no ipv6 multicast boundary scope"
 
     def update(self, old, upd, new, int_name):
@@ -371,7 +344,7 @@ _cvts.append(_Cvt_Int_IPv6MultBdry)
 class _Cvt_Int_IPv6PIMBSRBdr(_CvtContext_Int):
     cmd = "ipv6-pim", "bsr-border"
 
-    def remove(self, old, rem, int_name):
+    def remove(self, old, int_name):
         return "interface " + int_name, " no ipv6 pim bsr border"
 
     def update(self, old, upd, new, int_name):
@@ -381,19 +354,14 @@ _cvts.append(_Cvt_Int_IPv6PIMBSRBdr)
 
 
 class _Cvt_Int_IPv6TrafFilt(_CvtContext_Int):
-    cmd = "ipv6-traffic-filter",
+    cmd = "ipv6-traffic-filter", None
 
-    def remove(self, old, rem, int_name):
-        l = ["interface " + int_name]
-        for dir_ in sorted(rem):
-            l.append(" no ipv6 traffic-filter " + dir_)
-        return l
+    def remove(self, old, int_name, dir_):
+        return "interface " + int_name, " no ipv6 traffic-filter " + dir_
 
-    def update(self, old, upd, new, int_name):
-        l = ["interface " + int_name]
-        for dir_ in sorted(upd):
-            l.append(" ipv6 traffic-filter %s %s" % (upd[dir_], dir_))
-        return l
+    def update(self, old, upd, new, int_name, dir_):
+        return ("interface " + int_name,
+               " ipv6 traffic-filter %s %s" % (upd, dir_))
 
 _cvts.append(_Cvt_Int_IPv6TrafFilt)
 
@@ -411,9 +379,16 @@ _cvts.append(_Cvt_Int_IPv6VerifyUni)
 
 
 class _Cvt_Int_ServPol(_CvtContext_Int):
-    cmd = "service-policy",
+    cmd = "service-policy", None
 
-    def remove(self, old, rem, int_name):
+    # this uses truncate() to ensure that all removes happen before any
+    # update()s - this because an old policy must be removed before a
+    # new one of the same type/direction is added
+    #
+    # calls to remove() will be matched in alphabetical order which
+    # would match the add first, if it's earlier in the list
+
+    def truncate(self, old, rem, int_name):
         l = ["interface " + int_name]
         for policy in sorted(rem):
             l.append(" no service-policy " + policy)
@@ -431,7 +406,7 @@ _cvts.append(_Cvt_Int_ServPol)
 class _Cvt_Int_StandbyIP(_CvtContext_Int):
     cmd = "standby", "group", None, "ip"
 
-    def remove(self, old, rem, int_name, grp):
+    def remove(self, old, int_name, grp):
         return "interface " + int_name, " no standby %d ip" % grp
 
     def update(self, old, upd, new, int_name, grp):
@@ -441,37 +416,29 @@ _cvts.append(_Cvt_Int_StandbyIP)
 
 
 class _Cvt_Int_StandbyIPSec(_CvtContext_Int):
-    cmd = "standby", "group", None, "ip-secondary"
+    cmd = "standby", "group", None, "ip-secondary", None
 
-    def remove(self, old, rem, int_name, grp):
-        l = ["interface " + int_name]
-        for addr in sorted(rem):
-            l.append(" no standby %d ip %s secondary" % (grp, addr))
-        return l
+    def remove(self, old, int_name, grp, addr):
+        return ("interface " + int_name,
+                " no standby %d ip %s secondary" % (grp, addr))
 
-    def update(self, old, upd, new, int_name, grp):
-        l = ["interface " + int_name]
-        for addr in sorted(upd):
-            l.append(" standby %d ip %s secondary" % (grp, addr))
-        return l
+    def update(self, old, upd, new, int_name, grp, addr):
+        return ("interface " + int_name,
+                " standby %d ip %s secondary" % (grp, addr))
 
 _cvts.append(_Cvt_Int_StandbyIPSec)
 
 
 class _Cvt_Int_StandbyIPv6(_CvtContext_Int):
-    cmd = "standby", "group", None, "ipv6"
+    cmd = "standby", "group", None, "ipv6", None
 
-    def remove(self, old, rem, int_name, grp):
-        l = ["interface " + int_name]
-        for addr in sorted(rem):
-            l.append(" no standby %d ipv6 %s" % (grp, addr))
-        return l
+    def remove(self, old, int_name, grp, addr):
+        return ("interface " + int_name,
+                " no standby %d ipv6 %s" % (grp, addr))
 
-    def update(self, old, upd, new, int_name, grp):
-        l = ["interface " + int_name]
-        for addr in sorted(upd):
-            l.append(" standby %d ipv6 %s" % (grp, addr))
-        return l
+    def update(self, old, upd, new, int_name, grp, addr):
+        return ("interface " + int_name,
+                " standby %d ipv6 %s" % (grp, addr))
 
 _cvts.append(_Cvt_Int_StandbyIPv6)
 
@@ -479,7 +446,7 @@ _cvts.append(_Cvt_Int_StandbyIPv6)
 class _Cvt_Int_StandbyPreempt(_CvtContext_Int):
     cmd = "standby", "group", None, "preempt"
 
-    def remove(self, old, rem, int_name, grp):
+    def remove(self, old, int_name, grp):
         return "interface " + int_name, " no standby %d preempt" % grp
 
     def update(self, old, upd, new, int_name, grp):
@@ -491,7 +458,7 @@ _cvts.append(_Cvt_Int_StandbyPreempt)
 class _Cvt_Int_StandbyPri(_CvtContext_Int):
     cmd = "standby", "group", None, "priority"
 
-    def remove(self, old, rem, int_name, grp):
+    def remove(self, old, int_name, grp):
         return "interface " + int_name, " no standby %d priority" % grp
 
     def update(self, old, upd, new, int_name, grp):
@@ -503,7 +470,7 @@ _cvts.append(_Cvt_Int_StandbyPri)
 class _Cvt_Int_StandbyTimers(_CvtContext_Int):
     cmd = "standby", "group", None, "timers"
 
-    def remove(self, old, rem, int_name, grp):
+    def remove(self, old, int_name, grp):
         return "interface " + int_name, " no standby %d timers" % grp
 
     def update(self, old, upd, new, int_name, grp):
@@ -513,20 +480,14 @@ _cvts.append(_Cvt_Int_StandbyTimers)
 
 
 class _Cvt_Int_StandbyTrk(_CvtContext_Int):
-    cmd = "standby", "group", None, "track"
+    cmd = "standby", "group", None, "track", None
 
-    def remove(self, old, rem, int_name, grp):
-        l = ["interface " + int_name]
-        for trk in sorted(rem):
-            l.append(" no standby %d track %s" % (grp, trk))
-        return l
+    def remove(self, old, int_name, grp, obj):
+        return "interface " + int_name, " no standby %d track %s" % (grp, obj)
 
-    def update(self, old, upd, new, int_name, grp):
-        l = ["interface " + int_name]
-        for trk in sorted(upd):
-            l.append(" standby %d track %s" % (grp, trk)
-                     + (" %s" % upd[trk] if upd[trk] else ""))
-        return l
+    def update(self, old, upd, new, int_name, grp, obj):
+        return ("interface " + int_name,
+                " standby %d track %s%s" % (grp, obj, upd or ""))
 
 _cvts.append(_Cvt_Int_StandbyTrk)
 
@@ -534,7 +495,7 @@ _cvts.append(_Cvt_Int_StandbyTrk)
 class _Cvt_Int_StandbyVer(_CvtContext_Int):
     cmd = "standby", "version"
 
-    def remove(self, old, rem, int_name):
+    def remove(self, old, int_name):
         return "interface " + int_name, " no standby version"
 
     def update(self, old, upd, new, int_name):
@@ -546,7 +507,7 @@ _cvts.append(_Cvt_Int_StandbyVer)
 class _Cvt_Int_StormCtrl(_CvtContext_Int):
     cmd = "storm-control", None
 
-    def remove(self, old, rem, int_name, traffic):
+    def remove(self, old, int_name, traffic):
         return ("interface " + int_name,
                 " no storm-control %s level" % traffic)
 
@@ -560,7 +521,7 @@ _cvts.append(_Cvt_Int_StormCtrl)
 class _Cvt_Int_SwPort(_CvtContext_Int):
     cmd = "switchport",
 
-    def remove(self, old, rem, int_name):
+    def remove(self, old, int_name):
         # if the 'switchport' option is not present, that doesn't mean
         # it's disabled but just that it's not specified, so we assume
         # the default is for it to be disabled
@@ -580,7 +541,7 @@ _cvts.append(_Cvt_Int_SwPort)
 class _Cvt_Int_SwPortMode(_CvtContext_Int):
     cmd = "switchport-mode",
 
-    def remove(self, old, rem, int_name):
+    def remove(self, old, int_name):
         return "interface " + int_name, " no switchport mode"
 
     def update(self, old, upd, new, int_name):
@@ -592,7 +553,7 @@ _cvts.append(_Cvt_Int_SwPortMode)
 class _Cvt_Int_SwPortNoNeg(_CvtContext_Int):
     cmd = "switchport-nonegotiate",
 
-    def remove(self, old, rem, int_name):
+    def remove(self, old, int_name):
         return "interface " + int_name, " no switchport nonegotiate"
 
     def update(self, old, upd, new, int_name):
@@ -607,7 +568,7 @@ class _Cvt_Int_SwPortTrkNtv(_CvtContext_Int):
     cmd = tuple()
     ext = "switchport-trunk-native",
 
-    def remove(self, old, rem, int_name):
+    def remove(self, old, int_name):
         # if this interface is in a port-channel, we do all changes
         # there, so skip this
         if "channel-group" in old:
@@ -633,7 +594,7 @@ class _Cvt_Int_SwPortTrkAlw(_CvtContext_Int):
     cmd = tuple()
     ext = "switchport-trunk-allow",
 
-    def remove(self, old, rem, int_name):
+    def truncate(self, old, rem, int_name):
         # if this interface is in a port-channel, we do all changes
         # there, so skip this
         if "channel-group" in old:
@@ -661,7 +622,7 @@ _cvts.append(_Cvt_Int_SwPortTrkAlw)
 class _Cvt_Int_XConn(_CvtContext_Int):
     cmd = "xconnect",
 
-    def remove(self, old, rem, int_name):
+    def remove(self, old, int_name):
         return "interface " + int_name, " no xconnect"
 
     def update(self, old, upd, new, int_name):
@@ -677,7 +638,7 @@ class _Cvt_Int_NoShutdown(_CvtContext_Int):
     cmd = "shutdown",
     name = "no-shutdown"
 
-    def remove(self, old, rem, int_name):
+    def remove(self, old, int_name):
         return "interface " + int_name, " no shutdown"
 
 _cvts.append(_Cvt_Int_NoShutdown)
@@ -691,7 +652,7 @@ _cvts.append(_Cvt_Int_NoShutdown)
 class _Cvt_IPACL_Std(DiffConvert):
     cmd = "ip-access-list-standard", None
 
-    def remove(self, old, rem, acl_name):
+    def remove(self, old, acl_name):
         return "no ip access-list standard " + acl_name
 
     def update(self, old, upd, new, acl_name):
@@ -705,7 +666,7 @@ _cvts.append(_Cvt_IPACL_Std)
 class _Cvt_IPACL_Ext(DiffConvert):
     cmd = "ip-access-list-extended", None
 
-    def remove(self, old, rem, acl_name):
+    def remove(self, old, acl_name):
         return "no ip access-list extended " + acl_name
 
     def update(self, old, upd, new, acl_name):
@@ -719,7 +680,7 @@ _cvts.append(_Cvt_IPACL_Ext)
 class _Cvt_IPv6ACL_Ext(DiffConvert):
     cmd = "ipv6-access-list", None
 
-    def remove(self, old, rem, acl_name):
+    def remove(self, old, acl_name):
         return "no ipv6 access-list " + acl_name
 
     def update(self, old, upd, new, acl_name):
@@ -738,7 +699,7 @@ _cvts.append(_Cvt_IPv6ACL_Ext)
 class _Cvt_IPPfxList(DiffConvert):
     cmd = "ip-prefix-list", None
 
-    def remove(self, old, rem, pfx_name):
+    def remove(self, old, pfx_name):
         return "no ip prefix-list " + pfx_name
 
     def update(self, old, upd, new, pfx_name):
@@ -750,7 +711,7 @@ _cvts.append(_Cvt_IPPfxList)
 class _Cvt_IPv6PfxList(DiffConvert):
     cmd = "ipv6-prefix-list", None
 
-    def remove(self, old, rem, pfx_name):
+    def remove(self, old, pfx_name):
         return "no ipv6 prefix-list " + pfx_name
 
     def update(self, old, upd, new, pfx_name):
@@ -768,7 +729,7 @@ _cvts.append(_Cvt_IPv6PfxList)
 class _Cvt_IPRoute(DiffConvert):
     cmd = "ip-route", None
 
-    def remove(self, old, rem, route):
+    def remove(self, old, route):
         return "no ip route " + route
 
     def update(self, old, upd, new, route):
@@ -780,7 +741,7 @@ _cvts.append(_Cvt_IPRoute)
 class _Cvt_IPv6Route(DiffConvert):
     cmd = "ipv6-route", None
 
-    def remove(self, old, rem, route):
+    def remove(self, old, route):
         return "no ipv6 route " + route
 
     def update(self, old, upd, new, route):
@@ -797,7 +758,7 @@ _cvts.append(_Cvt_IPv6Route)
 class _Cvt_NoSTP(DiffConvert):
     cmd = "no-spanning-tree-vlan", None
 
-    def remove(self, old, rem, tag):
+    def remove(self, old, tag):
         # removing 'no spanning-tree' enables spanning-tree
         return "spanning-tree vlan %d" % tag
 
@@ -811,7 +772,7 @@ _cvts.append(_Cvt_NoSTP)
 class _Cvt_STPPri(DiffConvert):
     cmd = "spanning-tree-vlan-priority", None
 
-    def remove(self, old, rem, tag):
+    def remove(self, old, tag):
         return "no spanning-tree vlan %d priority" % tag
 
     def update(self, old, upd, new, tag):
@@ -829,7 +790,7 @@ class _Cvt_Track(DiffConvert):
     cmd = "track", None
     ext = "criterion",
 
-    def remove(self, old, rem, obj):
+    def remove(self, old, obj):
         return "no track %d" % obj
 
     def update(self, old, upd, new, obj):
@@ -845,7 +806,7 @@ class _CvtContext_Track(DiffConvert):
 class _Cvt_Track_Delay(_CvtContext_Track):
     cmd = "delay",
 
-    def remove(self, old, rem, obj):
+    def remove(self, old, obj):
         return "track %d" % obj, " no delay"
 
     def update(self, old, upd, new, obj):
@@ -857,7 +818,7 @@ _cvts.append(_Cvt_Track_Delay)
 class _Cvt_Track_IPVRF(_CvtContext_Track):
     cmd = "ip-vrf",
 
-    def remove(self, old, rem, obj):
+    def remove(self, old, obj):
         return "track %d" % obj, " no ip vrf"
 
     def update(self, old, upd, new, obj):
@@ -869,7 +830,7 @@ _cvts.append(_Cvt_Track_IPVRF)
 class _Cvt_Track_IPv6VRF(_CvtContext_Track):
     cmd = "ipv6-vrf",
 
-    def remove(self, old, rem, obj):
+    def remove(self, old, obj):
         return "track %d" % obj, " no ipv6 vrf"
 
     def update(self, old, upd, new, obj):
@@ -880,19 +841,13 @@ _cvts.append(_Cvt_Track_IPv6VRF)
 
 class _Cvt_Track_Obj(_CvtContext_Track):
     context = "track", None
-    cmd = "object",
+    cmd = "object", None
 
-    def remove(self, old, rem, obj):
-        l = ["track %d" % obj]
-        for obj in rem:
-            l.append(" no object " + obj)
-        return l
+    def remove(self, old, obj, sub_obj):
+        return "track %d" % obj, " no object " + sub_obj
 
-    def update(self, old, upd, new, obj):
-        l = ["track %d" % obj]
-        for obj in upd:
-            l.append(" object " + obj)
-        return l
+    def update(self, old, upd, new, obj, sub_obj):
+        return "track %d" % obj, " object " + sub_obj
 
 _cvts.append(_Cvt_Track_Obj)
 
@@ -905,12 +860,11 @@ _cvts.append(_Cvt_Track_Obj)
 class _Cvt_VLAN(DiffConvert):
     cmd = "vlan", None
 
-    def remove(self, old, rem, tag):
+    def remove(self, old, tag):
         return "no vlan %d" % tag
 
-    def update(self, old, upd, new, tag):
-        if upd and ("exists" in upd):
-            return "vlan %d" % tag
+    def add(self, new, tag):
+        return "vlan %d" % tag
 
 _cvts.append(_Cvt_VLAN)
 
@@ -919,7 +873,7 @@ class _Cvt_VLAN_Name(DiffConvert):
     context = "vlan", None
     cmd = "name",
 
-    def remove(self, old, rem, tag):
+    def remove(self, old, tag):
         return "vlan %d" % tag, " no name"
 
     def update(self, old, upd, new, tag):
