@@ -577,16 +577,18 @@ def explain_diffs(old, new, indent="", prefix=""):
     empty string for things like 'ip prefix-list ...', where they are
     not.
 
-    The 'prefix' is added after the indent and an explaining comment,
-    if there is one, followed by a line.  It is set to the empty string
-    for 'ip access-list ...' (where no extra string is required) or
-    something like 'ip prefix-list NAME', where these are required.
+    The 'prefix' is added after the indent, before the text of each
+    line.  It is not included on lines with comments.  It would be set
+    to the empty string for 'ip access-list ...' (where no extra string
+    is required) or something like 'ip prefix-list NAME ', where this
+    is required on the same line.  Note that it must contain a trailing
+    space, if one is required, between it and the line.
 
     If either of the two lists supplied are None, an empty list is
     assumed, either removing or adding all the entries.
 
     The return value is a list of lines these explaining comments
-    inserte4d.  The lines without comments will create the new list.
+    inserted.  The lines without comments will create the new list.
     """
 
     # initialise the returned list
@@ -603,7 +605,7 @@ def explain_diffs(old, new, indent="", prefix=""):
     for line in _differ.compare(old or [], new or []):
         op, text = line[0], line[2:]
 
-        if op ==" ":
+        if op == " ":
             # line is the same
 
             # if we were previously adding lines, indicate we're not
